#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests
import json
import logging
import os
import yaml
import pandas as pd
from jinja2 import Environment, FileSystemLoader
from endika_utils.validator_model_utils import validate_dict
logger = logging.getLogger(__name__)


class ACIObject:

    def __init__(self, url_base, admin_name, password):
        self.url_base = url_base
        self.admin_name = admin_name
        self.password = password
        self._token = self.get_token()

    def __str__(self):
        return f"""
      ACI Object Instance:
          URL_base: {self.url_base}
          Admin_name: {self.admin_name}
          Pass: ********
          Token: {self._token}
      """

    def get_token(self):
        url = self.url_base + "/api/aaaLogin.json"
        payload = {
            "aaaUser": {
                "attributes": {
                    "name": self.admin_name,
                    "pwd": self.password
                }
            }
        }
        headers = {
            "Content-Type": "application/json"
        }
        requests.packages.urllib3.disable_warnings()
        try:
            response = requests.post(url, data=json.dumps(
                payload), headers=headers, verify=False).json()
        except requests.exceptions.ConnectionError as c:
            print(c)
            logger.error("Connection Failed: " + str(c))
            exit()
        except BaseException as e:
            print(e)
            print('type is:', e.__class__.__name__)
            logger.error("Unhandled Exception: " + str(e))
            exit()

        if (response['imdata'][0]['aaaLogin']['attributes']['token']):
            return(response['imdata'][0]['aaaLogin']['attributes']['token'])
        else:
            logger.error("Unable to log: " + response["imdata"]
                         [0]["error"]["attributes"]["text"])
            exit()

# Funcion generica para crear objetos de cualquier tipo, depende del payload pasado
    def create_object(self, payload):
        url = self.url_base + "/api/mo/uni.json"
        headers = {
            "Cookie": f"APIC-Cookie={self._token}",
        }
        requests.packages.urllib3.disable_warnings()
        # En las plantillas hay que poner none como default en vez de "", se corrige aqui
        payload = self._dict_replace_value(payload, "none", "")
        payload = self._dict_replace_value(payload, "nan", "")
        print(payload)
        print("****************************************")
        response = requests.post(url, data=json.dumps(
            payload), headers=headers, verify=False)
        if (response.status_code == 200):
            if "name" in payload[list(payload.keys())[0]]["attributes"]:
                print("Successfully created " +
                      payload[list(payload.keys())[0]]["attributes"]["name"])
                logger.info("Successfully created " +
                            payload[list(payload.keys())[0]]["attributes"]["name"])
            else:
                print("Successfully associated: " +
                      payload[list(payload.keys())[0]]["attributes"]["dn"])
                logger.info("Successfully associated: " +
                            payload[list(payload.keys())[0]]["attributes"]["dn"])
        else:
            print("Issue with creating: " + str(response.status_code))
            print(json.loads(response.text)["imdata"]
                  [0]["error"]["attributes"]["text"])
            logger.error("Issue with creating: " + json.loads(response.text)["imdata"]
                         [0]["error"]["attributes"]["text"])

# Dos funcionaes para reemplazar none por "" en diccionarios anidados
    def _dict_replace_value(self, d, old, new):
        x = {}
        for k, v in d.items():
            if isinstance(v, dict):
                v = self._dict_replace_value(v, old, new)
            elif isinstance(v, list):
                v = self._list_replace_value(v, old, new)
            elif isinstance(v, str):
                v = v.replace(old, new)
            x[k] = v
        return x

    def _list_replace_value(self, l, old, new):
        x = []
        for e in l:
            if isinstance(e, list):
                e = self._list_replace_value(e, old, new)
            elif isinstance(e, dict):
                e = self._dict_replace_value(e, old, new)
            elif isinstance(e, str):
                e = e.replace(old, new)
            x.append(e)
        return x


def parse_data_excel(excel_file='data/datos_maestro.xlsx', templates_dir='templates/', save=False):
    all_dicts = {}
    for filename_full in os.listdir(templates_dir):
        if filename_full.endswith(".j2"):
            filename = filename_full[:-3]
            env = Environment(loader=FileSystemLoader('.'))
            template = env.get_template(templates_dir + filename_full)

            nodes = pd.read_excel(
                excel_file, sheet_name=filename, engine='openpyxl', na_values="null")
            nodes = nodes.where((pd.notnull(nodes)), None)
            nodes = nodes.to_dict(orient='records')
            # Al leer, el diccionario devuelve nan con valores no rellenos
            # Se eliminan para no tener que tratarlos mas adelante, si no estan, no estan
            nodes_2 = []
            for item in nodes:
                item_2 = {}
                for key, value in item.items():
                    if (str(value) != "nan" and str(value) != "None"):
                        item_2[key] = item[key]
                nodes_2.append(item_2)
            nodes = nodes_2
            print(nodes_2)
            if nodes != []:
                nodes_aux = []
                for item in nodes:
                    nodes_aux.append(validate_dict(filename, item))
                nodes = nodes_aux
                print(nodes)
            if save:
                with open("outputs/" + filename + ".yaml", "w") as w:
                    output = template.render(dataread=nodes)
                    w.write(output)
                w.close()


            # Versiones de pyaml anteriores a 6.0
            # all_dicts.update(yaml.load(template.render(dataread=nodes)))
            # Versiones de pyaml 6.0 y posteriores
            all_dicts.update(yaml.safe_load(template.render(dataread=nodes)))

    return all_dicts


if __name__ == '__main__':
    print("Estas ejecutando directamente aci_utils ")



#!/usr/bin/env

from pydantic import BaseModel, validator, constr, conint
from ipaddress import IPv4Address
from typing import Optional, Type, Literal
from pydantic import BaseModel as PydanticBaseModel
import re

# Extendemos la clase BaseModel por defecto con nuestra propia clase para hacer strip por defecto


class BaseModel(PydanticBaseModel):
    class Config:
        anystr_strip_whitespace = True

# Cada clase tiene las variables de su template, con los validadores necesarios
# Para validadores mas complejos, se puede añadir una funcion


class Aps_dict_model(BaseModel):
    """Clase para validar el formato del diccionario aps_dict"""
    name: constr(strict=True, regex=r'^aps-\S+$')
    lip: constr(strict=True, regex=r'^lip-\S+$')
    lpg: constr(strict=True, regex=r'^lpg-\S+$')
    from_port: conint(strict=True, gt=0, lt=56)
    to_port: conint(strict=True, gt=0, lt=56)
    description: Optional[str] = None

    @validator('to_port')
    def to_port_check(cls, to_port, values):
        if values['from_port'] > to_port:
            raise ValueError(
                "to_port tiene que ser un numero menor que from_port")
        else:
            return to_port


class Bd_dict_model(BaseModel):
    """Clase para validar el formato del diccionario tnt_dict"""
    bd: constr(strict=True, regex=r'^bd-\S+$')
    tenant: constr(strict=True, regex=r'^tnt-\S+$')
    vrf: constr(strict=True, regex=r'^vrf-\S+$')
    description: Optional[constr(max_length=70)]
    bd_alias: Optional[constr(strict=True, regex=r'^vlan-\d{1,4}$')] = None
    arp_flooding: Optional[Literal['yes', 'no']]
    enable_routing: Optional[Literal['yes', 'no']] = None
    l2_unknown_unicast: Optional[Literal['flood', 'proxy']] = None
    multi_dest: Optional[Literal['bd-flood', 'drop', 'encap-flood']] = None
    erp: Optional[constr(strict=True, regex=r'^erp-\S+$')] = None
    gateway: Optional[IPv4Address] = None
    mask: Optional[constr(strict=True)] = None
    scope: Optional[Literal['private', 'public', 'shared']] = None
    l3out: Optional[constr(strict=True, regex=r'^l30-\S+$')] = None

    @validator('bd_alias')
    def vlan_check(cls, bd_alias):
        vlan = re.search('(\d+)$', bd_alias)
        if int(vlan.group(1)) > 4096:
            raise ValueError("El valor de la vlan no puede ser mayor de 4096")
        else:
            return bd_alias

    #Hay que forzar a que lo devuelva como string, sino devuelve un objeto tipo IPv4Address
    @validator('gateway')
    def return_gateway(cls,gateway):
        return(str(gateway))

    #Funcion brujeria de internet, ni he intentado entenderla, pero he probado y funciona
    #https://codereview.stackexchange.com/questions/209243/verify-a-subnet-mask-for-validity-in-python
    @validator('mask')
    def mask_check(cls,mask):
        a, b, c, d = (int(octet) for octet in mask.split("."))
        mask_tmp = a << 24 | b << 16 | c << 8 | d
        if mask_tmp == 0:
            raise ValueError("0.0.0.0 no se acepta como mascara", mask)
        m = mask_tmp & -mask_tmp
        right0bits = -1
        while m:
            m >>=1
            right0bits +=1
        if mask_tmp | ((1 << right0bits) - 1) != 0xffffffff:
            raise ValueError(mask + " no es una mascara de red correcta")
        else:
            #return 32 - right0bits #devolveriamos la mascara como numero
            return mask


class Epg_dict_model(BaseModel):
    """Clase para validar el formato del diccionario tnt_dict"""
    epg: constr(strict=True, regex=r'^epg-\S+$')
    tenant: constr(strict=True, regex=r'^tnt-\S+$')
    apf: constr(strict=True, regex=r'^apf-\S+$')
    bd: constr(strict=True, regex=r'^bd-\S+$')
    floodOnEncap: Optional[Literal['disabled', 'enabled']] = None
    description: Optional[constr(max_length=70)]
    epg_alias: Optional[constr(strict=True, regex=r'^vlan-\d{1,4}$')] = None
    

    @validator('epg_alias')
    def vlan_check(cls, epg_alias):
        vlan = re.search('(\d+)$', epg_alias)
        if int(vlan.group(1)) > 4096:
            raise ValueError("El valor de la vlan no puede ser mayor de 4096")
        else:
            return epg_alias

class Tnt_dict_model(BaseModel):
    """Clase para validar el formato del diccionario tnt_dict"""
    tenant: constr(strict=True, regex=r'^tnt-\S+$')
    description: Optional[constr(strict=True)] = None


def validate_dict(dict_type, my_dict):
    """Clase para validar un diccionario, tiene que recibir el tipo de diccionario"""
    validated_obj = None
    if dict_type == 'aps':
        validated_obj = Aps_dict_model(**my_dict)
    elif dict_type == 'bd':
        validated_obj = Bd_dict_model(**my_dict)
    elif dict_type == 'epg':
        validated_obj = Epg_dict_model(**my_dict)
    elif dict_type == 'tnt':
        validated_obj = Tnt_dict_model(**my_dict)
    try:
        # Al crear las clases con atributos, el diccionario devuelve nan con valores no rellenos
        # Se eliminan para no tener que tratarlos mas adelante, si no estan, no estan
        for key, value in list(validated_obj.__dict__.items()):
            if value == None:
                validated_obj.__dict__.pop(key)
        return validated_obj.__dict__
    except AttributeError:
        raise Exception("No se ha encontrado el tipo de diccionario")
    del validated_obj


if __name__ == '__main__':
    print("Estas ejecutando directamente validator_mode_utils ")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='aci_utils',
    version='0.0.1',
    license='MIT',
    author="Endika Zuazo",
    author_email='endikazuazo@gmail.com',
    description="Define una clase para interactuar con la API de Cisco ACI",
    packages=setuptools.find_packages(),
    keywords='example project',
)